unit NClipboard;

{$IMPORTEDDATA OFF}

interface

uses Windows;

function ReadClipboard: WideString; overload;
function ReadClipboard(var Data: Pointer): Cardinal; overload;
procedure WriteClipboard(const S: WideString; Owner: HWND=0);

implementation

uses NSys;

function ReadClipboard: WideString;
var
  nbytes: Integer;
  h: THandle;
  p: Pointer;
begin
  if not OpenClipboard(0) then
    raise EOSError.Create;
  h := 0;
  p := nil;
  Result := '';
  try
    h := GetClipboardData(CF_UNICODETEXT);
    if h = 0 then
      Exit;
    p := GlobalLock(h);
    if p = nil then
      raise EOSError.Create;
    nbytes := GlobalSize(h);
    if nbytes = 0 then
      Exit;
    Dec(nbytes, SizeOf(WideChar)); // for terminating null
    SetLength(Result, nbytes div SizeOf(WideChar));
    Move(p^, Result[1], nbytes);
  finally
    if p <> nil then
      GlobalUnlock(h);
    CloseClipboard;
  end;
end;

function ReadClipboard(var Data: Pointer): Cardinal;
const
  headersize = SizeOf(TBitmapFileHeader) + SizeOf(TBitmapInfoHeader);
var
  h: THandle;
  bm: TBitmap;
  pbmi: PBitmapInfo;
  dc: HDC;
begin
  if not OpenClipboard(0) then
    raise EOSError.Create;
  Data := nil;
  Result := 0;
  try
    h := GetClipboardData(CF_BITMAP);
    if h = 0 then
      Exit;
    if GetObject(h, SizeOf(bm), @bm) < SizeOf(bm) then
      Exit;
    Result := headersize + bm.bmWidthBytes * bm.bmHeight;
    GetMem(Data, Result);
    FillChar(Data^, headersize, 0);
    with PBitmapFileHeader(Data)^ do begin
      bfType := Byte('B') + Byte('M') shl 8;
      bfOffBits := headersize;
      bfSize := Result;
    end;
    pbmi := @PChar(Data)[SizeOf(TBitmapFileHeader)];
    with pbmi.bmiHeader do begin
      biSize := SizeOf(pbmi.bmiHeader);
      biWidth := bm.bmWidth;
      biHeight := bm.bmHeight;
      biPlanes := 1;
      biBitCount := 24;
      biCompression := BI_RGB;
    end;
    dc := GetDC(0);
    if (dc = 0) or(GetDIBits(dc, h, 0, bm.bmHeight, @PChar(Data)[headersize],
        pbmi^, DIB_RGB_COLORS) = 0) then begin
      FreeMem(Data);
      Data := nil;
      Result := 0;
      Exit;
    end;
  finally
    CloseClipboard;
  end;
end;

procedure WriteClipboard(const S: WideString; Owner: HWND=0);
var
  nbytes: Integer;
  h: THandle;
  p: Pointer;
begin
  if not OpenClipboard(Owner) then
    raise EOSError.Create;
  h := 0;
  p := nil;
  try
    nbytes := (Length(S) + 1) * SizeOf(WideChar); // +1 for terminating null
    h := GlobalAlloc(GMEM_MOVEABLE, nbytes);
    if h = 0 then
      raise EOSError.Create;
    p := GlobalLock(h);
    if p = nil then
      raise EOSError.Create;
    Move(S[1], p^, nbytes);
    if not EmptyClipboard then
      raise EOSError.Create;
    if SetClipboardData(CF_UNICODETEXT, h) = 0 then
      raise EOSError.Create;
  finally
    if p <> nil then
      GlobalUnlock(h);
    CloseClipboard;
  end;
end;

end.
